/********************************************************************************
 *                                  DISCLAIMER                                  *
 * This software and documentation (the "Software") were developed at the Food  *
 * and Drug Administration (FDA) by employees of the Federal Government in the  *
 * course of their official duties. Pursuant to Title 17, Section 105 of the    *
 * United States Code, this work is not subject to copyright protection and is  *
 * in the public domain. Permission is hereby granted, free of charge, to any   *
 * person obtaining a copy of the Software, to deal in the Software without     *
 * restriction, including without limitation the rights to use, copy, modify,   *
 * merge, publish, distribute, sublicense, or sell copies of the Software or    *
 * derivatives, and to permit persons to whom the Software is furnished to do   *
 * so. FDA assumes no responsibility whatsoever for use by other parties of     *
 * the Software, its source code, documentation or compiled executables, and    *
 * makes no guarantees, expressed or implied, about its quality, reliability,   *
 *  or any other characteristic. Further, use of this code in no way implies    *
 * endorsement by the FDA or confers any advantage in regulatory decisions.     *
 * Although this software can be redistributed and/or modified freely, we       *
 * ask that any derivative works bear some notice that they are derived from    *
 * it, and any modified versions bear some notice that they have been modified. * 
 *                                                                              *
 * Written by:                                                                  *
 *  Lars Johannesen                                                             *
********************************************************************************/
#include "mex.h"

#include <ecglib>

using namespace arma;
mat calcU3(const mat &dat, const double fs, int U3N=100, int ith=2) {
	using namespace ecglib;

	int nsamples = dat.n_rows;
	int nleads = dat.n_cols;

	mat out(nsamples,nleads);
	U3N = static_cast<int>(round(fs/1000.0*U3N));

	for(int i = 0; i < nleads; ++i) {
		calcU3(dat.begin_col(i),dat.end_col(i),out.begin_col(i),U3N,ith);
	}

	return out;
}

template<typename TYP>
TYP get_matlab_scalar(const mxArray *a) {
	TYP *t = (TYP*)mxGetData(a);
	return *t;
}

bool isDoubleScalar(const mxArray *in) {
        return mxIsDouble(in) && mxGetNumberOfElements(in)==1;
}

void mexFunction (int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	if(nrhs != 4) 
		mexErrMsgTxt("Need 3 arguments data, fs, u3n, ith");

	using namespace arma;
	using namespace ecglib;
	
	try {
		if(!mxIsDouble(prhs[0]))
			mexErrMsgTxt("First argument must be a struct");

		if(!isDoubleScalar(prhs[1]))
			mexErrMsgTxt("Second argument must be a double scalar");

		if(!isDoubleScalar(prhs[2]))
			mexErrMsgTxt("Third argument must be a double scalar");

		if(!isDoubleScalar(prhs[3]))
			mexErrMsgTxt("Third argument must be a double scalar");

		const double *datptr = (const double*) mxGetPr(prhs[0]);
		int m = mxGetN(prhs[0]);
		int n = mxGetM(prhs[0]);
		mat indat(datptr, n, m);

		int u3n = get_matlab_scalar<double>(prhs[2]);
		int ith = get_matlab_scalar<double>(prhs[3]);

		double fs = get_matlab_scalar<double>(prhs[1]);
		mat outmat = calcU3(indat, fs, u3n, ith);
		
		mwSize dims[2];
		dims[0] = outmat.n_rows;
		dims[1] = outmat.n_cols;
		mxArray *a = mxCreateNumericArray(2, dims, mxDOUBLE_CLASS, mxREAL);
		double *dst = (double*)mxGetData(a);
		memcpy(dst,outmat.memptr(), dims[0]*dims[1]*sizeof(double));

		plhs[0] = a;
	} catch (std::logic_error &exp) {
		mexErrMsgTxt(exp.what());
	} catch (...) {
		mexErrMsgTxt("Unknown error");
	}
}
