/**
 * @file util.hpp
 * @author Lars Johannesen <lj@halloffame.dk>
 * @version 1.0
 *
 * @section LICENSE
 * This program is licensed under LGPL3, see LICENSE
 * 
 * @section DESCRIPTION
 * This file defines ecglib exception class
 **/
#ifndef ECGLIB_UTIL_LJ_2011_06_12
#define ECGLIB_UTIL_LJ_2011_06_12 1

#include <ecglib_bits/ecglib.hpp>

namespace ecglib { 
	/**
	 * @brief Converts sample to time
	 *
	 * @param sample Sample number
	 * @param fs Sampling frequency
	 *
	 * @return Time in ms
	 */
	inline timems sample_to_time(const double sample, const double fs) {
		return static_cast<unsigned int>(round(1000./fs * sample));
	}

	/**
	 * @brief Converts time to sample
	 *
	 * @param time Time in ms
	 * @param fs Sampling rate
	 *
	 * @return Sample
	 */
	inline sample time_to_sample(const timems time, const double fs) {
		return static_cast<unsigned int>(round(fs/1000. * time));
	}
}

#endif
