%% @file
% make_series.m
%
% @author Lars Johannesen and Loriano Galeotti
%
% @section DISCLAIMER
% This software and documentation (the "Software") were developed at the Food
% and Drug Administration (FDA) by employees of the Federal Government in the
% course of their official duties. Pursuant to Title 17, Section 105 of the
% United States Code, this work is not subject to copyright protection and is
% in the public domain. Permission is hereby granted, free of charge, to any
% person obtaining a copy of the Software, to deal in the Software without
% restriction, including without limitation the rights to use, copy, modify,
% merge, publish, distribute, sublicense, or sell copies of the Software or
% derivatives, and to permit persons to whom the Software is furnished to do
% so. FDA assumes no responsibility whatsoever for use by other parties of
% the Software, its source code, documentation or compiled executables, and
% makes no guarantees, expressed or implied, about its quality, reliability,
%  or any other characteristic. Further, use of this code in no way implies
% endorsement by the FDA or confers any advantage in regulatory decisions.
% Although this software can be redistributed and/or modified freely, we
% ask that any derivative works bear some notice that they are derived from
% it, and any modified versions bear some notice that they have been modified.
%
% @section Examples:
%  [ts] = make_series(anns, wavf, len, weight, flexible)
%
% @section See also:
%

%%
% @brief Creates a vector with wavf at each annotation
% @param anns Annotations in samples
% @param wavf Waveform
% @param len Length in samples
% @param weight Weights
% @param flexible Grow outside
% @return [ts] = Time series (with unit ms)
%
function [ts] = make_series(anns,wavf,len,weight,flexible)
%make_series Make series
%
% make_series(anns, fs, wavf, len,flexible)
%
% anns Annotations in samples
% fs Sampling frequency
% wavf Waveform
% len Length in samples
% weight annotation weight (optional)
% flexible (optional, default = 0) if =1 the output is allowed to grow over 'len' in case of annotations outside len 
%
%                                  DISCLAIMER
% This software and documentation (the "Software") were developed at the Food
% and Drug Administration (FDA) by employees of the Federal Government in the
% course of their official duties. Pursuant to Title 17, Section 105 of the
% United States Code, this work is not subject to copyright protection and is
% in the public domain. Permission is hereby granted, free of charge, to any
% person obtaining a copy of the Software, to deal in the Software without
% restriction, including without limitation the rights to use, copy, modify,
% merge, publish, distribute, sublicense, or sell copies of the Software or
% derivatives, and to permit persons to whom the Software is furnished to do
% so. FDA assumes no responsibility whatsoever for use by other parties of
% the Software, its source code, documentation or compiled executables, and
% makes no guarantees, expressed or implied, about its quality, reliability,
%  or any other characteristic. Further, use of this code in no way implies
% endorsement by the FDA or confers any advantage in regulatory decisions.
% Although this software can be redistributed and/or modified freely, we
% ask that any derivative works bear some notice that they are derived from
% it, and any modified versions bear some notice that they have been modified.
%
% Written by: Lars Johannesen and Loriano Galeotti
%
%
% Examples:
%  [ts] = make_series(anns, wavf, len)
%
% See also:

% LG: added weight option;
% improved range check for vector indexes;
% "merge" waves if they touch each other.
% removed limitation to use only even windows
% ensure the waveform is a col
% added flexible option (default off)
%
if nargin <6
    flexible = 0 ;
end
if nargin <5
    weight= [];
end
if length(weight)~=length(anns) % LG
    weight=ones(size(anns));
end

ts = zeros(len,1); % LG
wavf= wavf(:); % ensure the waveform is a col
nwavf = length(wavf);
left  = ceil(nwavf/2);
right  = nwavf-left;

for ii=1:length(anns)
    starti = max(anns(ii)-left+1,1); %LG
    if flexible
        stopi = (anns(ii)+right); % LG
    else
        stopi = min(anns(ii)+right , len); % LG
    end
    startw = max(1,-anns(ii)+left+1); % LG
%     stopw = min(nwavf, stopi-starti+1); %LG
    stopw = min(nwavf,stopi-starti+startw); %LG

    
    % 		startwav = 1;
    % 		stopwav = nwavf;
    
    %LG		if starti > 1 && stopi < len
    try
        ts(starti:stopi) = ts(starti:stopi) + wavf(startw:stopw) *weight(ii);
    catch
    	% FIXME Better to check bounds
        % fprintf('@@');
    end
    %LG		end
end
end
