//                                  DISCLAIMER                                  
// This software and documentation (the "Software") were developed at the Food  
// and Drug Administration (FDA) by employees of the Federal Government in the  
// course of their official duties. Pursuant to Title 17, Section 105 of the    
// United States Code, this work is not subject to copyright protection and is  
// in the public domain. Permission is hereby granted, free of charge, to any   
// person obtaining a copy of the Software, to deal in the Software without     
// restriction, including without limitation the rights to use, copy, modify,   
// merge, publish, distribute, sublicense, or sell copies of the Software or    
// derivatives, and to permit persons to whom the Software is furnished to do   
// so. FDA assumes no responsibility whatsoever for use by other parties of     
// the Software, its source code, documentation or compiled executables, and    
// makes no guarantees, expressed or implied, about its quality, reliability,   
//  or any other characteristic. Further, use of this code in no way implies    
// endorsement by the FDA or confers any advantage in regulatory decisions.     
// Although this software can be redistributed and/or modified freely, we       
// ask that any derivative works bear some notice that they are derived from    
// it, and any modified versions bear some notice that they have been modified. 
//                                                                              
// Written by: Lars Johannesen

#ifndef matconfig 
#define matconfig 1

#include <mex.h>

#include <ecglib>

namespace cinc2014 {
	template<typename TYP>
	TYP get_matlab_scalar(const mxArray *a) {
		double *t = (double*)mxGetData(a);
		TYP tt = static_cast<TYP>(*t);
		return tt;
	}

	template<typename TYP>
	mxArray *set_matlab_scalar(TYP t, mxClassID cla=mxDOUBLE_CLASS) {
		mwSize dims[2] = {1,1};
		mxArray *o = mxCreateNumericArray(2, dims, cla, mxREAL);
		double *d = (double*)mxGetData(o);
		*d=t;
		return o;
	}

	using namespace ecglib;

	class configfrom_matlab {
		public:
			configfrom_matlab(const mxArray *mx) : _mx(mx) {
				//FIXME: Check is struct
			}

		public:
			boost::any get(const std::string &nam, const Type typ) const {
				const mxArray *ptr = mxGetField(_mx, 0, nam.c_str());

				if(typ == Type::String) {
					char *str = mxArrayToString(ptr);
					std::string val(str);
					mxFree(str);
					return val;
				} else if(typ == Type::Double) {
					double val = get_matlab_scalar<double>(ptr);
					return val;
				} else if(typ == Type::Int) {
					int val = get_matlab_scalar<int>(ptr);
					return val;
				} else if(typ == Type::Uint) {
					unsigned int val = get_matlab_scalar<unsigned int>(ptr);
					return val;
				} else {
					throw std::logic_error("Unsupported type");
				}
			}
		
		private:
			const mxArray *_mx;
	};

	class configto_matlab {
		public:
			configto_matlab() {
			}

		public:
			void set(const std::string &nam, const property &prop) {
				if(prop.type == Type::String) {
					std::string val = boost::any_cast<std::string>(prop.value);
					_opts[nam] = mxCreateString(val.c_str());
				} else if(prop.type == Type::Double) {
					double val = boost::any_cast<double>(prop.value);
					_opts[nam] = set_matlab_scalar(val);
				} else if(prop.type == Type::Int) {
					int val = boost::any_cast<int>(prop.value);
					_opts[nam] = set_matlab_scalar(val);
				} else if(prop.type == Type::Uint) {
					unsigned int val = boost::any_cast<unsigned int>(prop.value);
					_opts[nam] = set_matlab_scalar(val);
				} else {
					throw std::logic_error("Unsupported type");
				}
			}

			mxArray *opts() const {
				std::vector<char*> keys;
				for(auto &r : _opts) {
					char *ptr = new char[r.first.size()+1];
					strcpy(ptr, r.first.c_str());
					keys.push_back(ptr);
				}

				const char **keyptr = const_cast<const char**>(&keys[0]);

				mwSize dims[1];
				dims[0] = 1;
				mxArray *opt = mxCreateStructArray(1, dims, keys.size(), keyptr);
				for(auto &r : _opts) {
					mxSetField(opt, 0, r.first.c_str(), r.second);
				}

				return opt;
			}

		private:
			std::map<std::string, mxArray*> _opts;
	};
}

#endif
