%% @file
% sv_detector.m
%
% @author Christopher Scully
%
% @section DISCLAIMER
% This software and documentation (the "Software") were developed at the Food
% and Drug Administration (FDA) by employees of the Federal Government in the
% course of their official duties. Pursuant to Title 17, Section 105 of the
% United States Code, this work is not subject to copyright protection and is
% in the public domain. Permission is hereby granted, free of charge, to any
% person obtaining a copy of the Software, to deal in the Software without
% restriction, including without limitation the rights to use, copy, modify,
% merge, publish, distribute, sublicense, or sell copies of the Software or
% derivatives, and to permit persons to whom the Software is furnished to do
% so. FDA assumes no responsibility whatsoever for use by other parties of
% the Software, its source code, documentation or compiled executables, and
% makes no guarantees, expressed or implied, about its quality, reliability,
%  or any other characteristic. Further, use of this code in no way implies
% endorsement by the FDA or confers any advantage in regulatory decisions.
% Although this software can be redistributed and/or modified freely, we
% ask that any derivative works bear some notice that they are derived from
% it, and any modified versions bear some notice that they have been modified.
%

%%
% @brief Returns an array of annotations for a continuous stroke volume signal
% @param sv_sig stroke volume signal sampled at 250 Hz
% @return anns annotations with the location of detected beats in ms
%
function [anns,sqi] = sv_detector(sv_sig)
%sv_detector Returns an array of annotations for a continuous stroke volume signal
%
% sv_sig stroke volume signal sampled at 250 Hz
% anns annotations with the location of detected beats in ms
%
%                                  DISCLAIMER                                  
% This software and documentation (the "Software") were developed at the Food  
% and Drug Administration (FDA) by employees of the Federal Government in the  
% course of their official duties. Pursuant to Title 17, Section 105 of the    
% United States Code, this work is not subject to copyright protection and is  
% in the public domain. Permission is hereby granted, free of charge, to any   
% person obtaining a copy of the Software, to deal in the Software without     
% restriction, including without limitation the rights to use, copy, modify,   
% merge, publish, distribute, sublicense, or sell copies of the Software or    
% derivatives, and to permit persons to whom the Software is furnished to do   
% so. FDA assumes no responsibility whatsoever for use by other parties of     
% the Software, its source code, documentation or compiled executables, and    
% makes no guarantees, expressed or implied, about its quality, reliability,   
%  or any other characteristic. Further, use of this code in no way implies    
% endorsement by the FDA or confers any advantage in regulatory decisions.     
% Although this software can be redistributed and/or modified freely, we       
% ask that any derivative works bear some notice that they are derived from    
% it, and any modified versions bear some notice that they have been modified. 
%                                                                              
% Written by: Christopher Scully

X=50;%if negative segment shorter than X, remove
Fs=250;

%bandpass filter signal
[bhp,ahp] = butter(5, [0.75]/(Fs/2.),'high');
yHP = filtfilt(bhp, ahp, sv_sig);
[blp,alp] = butter(5, [10]/(Fs/2.));
yLP = filtfilt(blp, alp, yHP);

%find (-) points
kneg=find(yLP<0);
%find discontinuities in (-) points
ndiff=diff(kneg);
kdisc=find(ndiff>1);
kdisc(diff(kdisc)<X)=[];
%find minimum for each section
locs=zeros(length(kdisc)+1,1);
s1=1;
s2=kdisc(1);
sect=yLP(kneg(s1):kneg(s2));
[~,ll]=min(sect);
locs(1)=ll+kneg(s1)-1;
for n=1:length(kdisc)-1
    s1=kdisc(n)+1;
    s2=kdisc(n+1);
    sect=yLP(kneg(s1):kneg(s2));
    [~,ll]=min(sect);
    locs(n+1)=ll+kneg(s1)-1;

end
s1=kdisc(end)+1;
sect=yLP(kneg(s1):end);
[~,ll]=min(sect);
locs(n+2)=ll+kneg(s1)-1;

%adjust sample locations to pulse onset time
anns=locs;
sqi = ones(size(anns));

end
